import requests
import json
import datetime


class DockerHubConfig:
    def __init__(
            self,
            dockerhub_user,
            dockerhub_password,
            pull_expire_date=None,
            push_expire_date=None
    ):
        self.url = 'https://hub.docker.com'
        self.auth_url = 'https://hub.docker.com/v2/users/login'
        self.dockerhub_user = dockerhub_user
        self.dockerhub_password = dockerhub_password
        self.token = self.__get_token(self.dockerhub_user, self.dockerhub_password)
        self.auth_headder = {'Authorization': f'JWT {self.token}'}
        self.pull_expire_date = pull_expire_date or datetime.datetime.now() - datetime.timedelta(days=10, )
        self.push_expire_date = push_expire_date or datetime.datetime.now() - datetime.timedelta(days=10, )

    def __get_token(self, username: str, password: str):
        token = ''
        if username and password:
            headers = {'Content-Type': 'application/json', }
            data = '{"username": ' + f'"{username}"' + f', "password": "{password}"' + '}'
            response = requests.post(self.auth_url, data=data, headers=headers)
            if response.status_code == 200:
                token = json.loads(response.text)['token']

        return token



import xml.etree.ElementTree as ET


class Xml:

    def __init__(self):
        self.xml_by_path = {}

    def get_variable_from_file(self, var_path, var_name, file):
        if file in self.xml_by_path:
            xml_content = self.xml_by_path[file]
        else:
            xml_content = ET.parse(file).getroot()
            self.xml_by_path[file] = xml_content

        var_value = None
        for type_tag in xml_content.findall(var_path):
            var_value = type_tag.get(var_name)

        if var_value is None:
            raise Exception('Var name: {} by path: {} not found.'.format(var_name, var_path))

        return var_value


import click
from aheadworks_composer_manager.model.extension import Extension
import requests
import os
import json

class ApiError(click.ClickException):
    pass

def api_call(ctx, method, endpoint, *args, **kwargs):
    headers = {'Content-Type': 'application/json',
               'Authorization': 'Bearer {0}'.format(ctx.obj['REPO_TOKEN'])}

    req = requests.Request(method, ctx.obj['REPO_FULL_URL'] + endpoint, headers=headers, *args, **kwargs)
    prepared = req.prepare()

    if ctx.obj['PRINT_REQUESTS']:
        click.echo("[DEBUG] >>> ({method}) {req.url} {req.body}".format(method=method.upper(), req=prepared, args=args, kwargs=kwargs))

    s = requests.Session()
    resp = s.send(prepared)

    if ctx.obj['PRINT_REQUESTS']:
        click.echo("[DEBUG] <<< (HTTP {resp.status_code}) {resp.text}".format(resp=resp))

    try:
        res = resp.json()
        if 'error' in res:
            raise ApiError(res['error'])
        return res
    except json.decoder.JSONDecodeError as e:
        if resp.text:
            raise ApiError("{}\n{}".format(e, resp.text))
        return {}


@click.group()
@click.pass_context
@click.option('--repo-url', envvar="REPO_URL", required=True)
@click.option('--repo-login', envvar="REPO_LOGIN", required=True)
@click.option('--repo-token', envvar="REPO_TOKEN", required=True)
@click.option('-p','--print-requests', is_flag=True, help="Verbosely print API requests")
def cli(ctx, repo_url, repo_login, repo_token, print_requests):
    ctx.obj = {}
    ctx.obj['REPO_FULL_URL'] = 'https://' + repo_login + '@' + repo_url
    ctx.obj['REPO_URL'] = repo_url
    ctx.obj['REPO_LOGIN'] = repo_login
    ctx.obj['REPO_TOKEN'] = repo_token
    ctx.obj['PRINT_REQUESTS'] = print_requests


@cli.command()
@click.pass_context
def get_packages(ctx):
    result = api_call(ctx, 'get', '/packages')
    click.echo(result)


@cli.command()
@click.pass_context
@click.argument('name')
def find_package(ctx, name):
    result = api_call(ctx, 'get', '/packages', params={'name': name})
    click.echo(result)


@cli.command()
@click.pass_context
@click.argument('name')
@click.argument('description', required=True)
def create_package(ctx, name, description):
    data = {
        'name': name,
        'description': description
    }
    result = api_call(ctx, 'post', '/packages', json=data)
    click.echo(result)


@cli.command()
@click.pass_context
@click.argument('package_zip', type=click.Path(exists=True, dir_okay=False))
@click.option('--status', default=1)
@click.option('--filename', required=False, help="Specify if filename is different at server")
def send_package(ctx, package_zip, status, filename):

    with Extension(package_zip) as e:
        create_package_data = {
            "name": e.meta.name,
            "description": e.meta.description,
            "status": 1
        }

        create_version_data = {
            'name': e.meta.version,
            'package': e.meta.name,
            'status': status,
            'filename': os.path.basename(filename or package_zip)
        }


        package_exists = api_call(ctx, 'get', '/packages', params={'name': e.meta.name})
        if len(package_exists):
            res = api_call(ctx, 'put', '/packages/%s' % package_exists[0]["id"], json=create_package_data)
        else:
            res = api_call(ctx, 'post', '/packages', json=create_package_data)

        version_exists = api_call(ctx, 'get', '/versions', params={'package': e.meta.name, 'name': e.meta.version})

        if len(version_exists):
            res = api_call(ctx, 'put', '/versions/%s' % version_exists[0]['id'], json=create_version_data)
            click.echo("Updated version {v[name]}".format(v=create_version_data))
        else:
            res = api_call(ctx, 'post', '/versions', json=create_version_data)
            click.echo("Created version {v[name]}".format(v=create_version_data))


@cli.command()
@click.pass_context
@click.argument('email', required=True)
@click.argument('public_key', required=True)
@click.argument('private_key', required=True)
def create_consumer(ctx, email, public_key, private_key):
    api_call(ctx, 'get', '/consumers')
    res = api_call(ctx, 'post', '/consumers', json={'email': email, 'public_key': public_key, 'private_key': private_key})
    click.echo("Created consumer #{res[id]} for email {res[email]}, pubkey {res[public_key]}".format(res=res))

@cli.command()
@click.pass_context
@click.argument('email', required=True)
@click.argument('public_key', required=True)
@click.argument('private_key', required=True)
def set_key(ctx, email, public_key, private_key):

    """Get consumer key(s)"""
    params = {'email': email}
    consumers = api_call(ctx, 'get', '/consumers', params=params)

    for c in consumers:
        cid = c['id']
        rq = {
            'email': email,
            'public_key': public_key,
            'private_key': private_key,
            'source': c['source'],
            'reference': c['reference']
        }
        res = api_call(ctx, 'put', '/consumers/%s' % cid,
                       json=rq)
        click.echo("Updated consumer #%s" % cid)
        return
    click.echo("No consumer match for email %s" % email, err=True)


@cli.command()
@click.pass_context
@click.argument('email', nargs=-1)
@click.option('--format', required=False)
def find_consumer(ctx, email, format):
    """Get consumer key(s)"""
    params = {'email': email}
    consumers = api_call(ctx, 'get', '/consumers', params=params)
    if not format:
        click.echo(consumers)
        return

    for c in consumers:
        click.echo(format.format(**c))


@cli.command()
@click.pass_context
@click.argument('email', required=True)
@click.argument('public_key', required=False, default='')
def delete_consumer(ctx, email, public_key):
    """Delete consumer key(s)"""
    params = {'email': email}
    if public_key:
        params.update({'public_key': public_key})
    consumers = api_call(ctx, 'get', '/consumers', params=params)

    n = 0
    for c in consumers:
        api_call(ctx, 'delete', '/consumers/%s' % c['id'])
        n += 1

    click.echo("Deleted {n} keys".format(n=n))


@cli.command()
@click.pass_context
@click.argument('package', required=True)
@click.argument('consumer', required=True, nargs=-1)
@click.option('--version', required=False, default='*')
def grant(ctx, package, consumer, version):
    """Give access to package to consumers"""
    ids = []

    for c in consumer:
        if '@' in c:
            found_consumers = [x['id'] for x in api_call(ctx, 'get', '/consumers', params={'email': c})]
            if not len(found_consumers):
                click.echo("Can't find consumer {}, skipping".format(c), err=True)
        else:
            found_consumers = [c]

        ids += found_consumers

    for cid in ids:
        # Delete permission if exists
        try:
            permissions = [p['id'] for p in api_call(ctx, 'get', '/permissions', params={'consumer_id': cid, 'package': package, 'version': version})]
            aaa = [api_call(ctx, 'delete', '/permissions/%s' % x) for x in permissions]
        except ApiError as e:
            pass
        api_call(ctx, 'post', '/permissions', json={'consumer_id': cid, 'package': package, 'version': version})

@cli.command()
@click.pass_context
@click.argument('consumer', required=True, nargs=-1)
def get_permissions(ctx, consumer):
    """Give access to package to consumers"""
    ids = []

    for c in consumer:
        if '@' in c:
            found_consumers = [x['id'] for x in api_call(ctx, 'get', '/consumers', params={'email': c})]
            if not len(found_consumers):
                click.echo("Can't find consumer {}, skipping".format(c), err=True)
        else:
            found_consumers = [c]

        ids += found_consumers

    for cid in ids:
        # Delete permission if exists
        try:
            permissions = [p for p in api_call(ctx, 'get', '/permissions', params={'consumer_id': cid})]
            click.echo(permissions)
        except ApiError as e:
            pass

if __name__ == '__main__':
    cli()



import json
import pathlib
import os
from os.path import join as _
import re
import shutil
import subprocess
import time
from aheadworks_core.api.composer_manager import ComposerManager
from aheadworks_core.model.cd import Cd as cd

class Console:
    def __init__(self):
        os.system("mkdir -p test-results")
        self.BASIC_PATH = pathlib.Path(os.environ.get('MAGENTO_ROOT', '/var/www/html'))
        ComposerManager.init_extra_repos()

    def assert_path_exists(self, path):
        if not os.path.exists(path):
            print(f"path '{path}' should exist", path)
            exit(1)

    def di_compile(self):
        # some modules in di:compile require active mysql
        try:
            os.system("/usr/bin/mysqld --user=root --console &>/dev/null &")
        except Exception as e:
            print(e)
        time.sleep(3)
        proc = subprocess.run([_(self.BASIC_PATH, 'bin/magento'), 'module:enable', '--all'])
        ec3 = proc.returncode
        proc = subprocess.run([_(self.BASIC_PATH, 'bin/magento'), 'setup:di:compile'])
        ec4 = proc.returncode
        print(f"module:enable ended with code {ec3}")
        print(f"di:compile ended with code {ec4}")


        if ec3 or ec4:
            print("Failed to di:compile")
            exit(1)

    def install(self, path):
        """
        Install extension(s) to path from path or zip
        """
        print("Installing from %s" % path)

        with open(_(path, 'composer.json')) as f:
            composer = json.load(f)
            # repo_name too hard for simple action
            repo_name = re.sub(r'[^a-z0-9_]', '_', composer['name'])

        with cd(self.BASIC_PATH):
            # here we download our aheadworks modules, that are in require in composer.json
            subprocess.run(
                ['composer', 'config', 'repositories.aheadworks', 'composer', 'https://composer.do.staging-box.net'])

            if os.getenv("COMPOSER_AUTH") is None:
                subprocess.run(['composer', 'config', 'http-basic.repo.magento.com', 'bf08744e4b4b3aee1d54fcd7cd56194a',
                            'f5b232eab5158a4597ecb00f8cacdf4a'])
            # main module we download always from path(the fastest way to test the latest version)
            subprocess.run(['composer', 'config', 'repositories.' + repo_name, 'path', path])

            # aheadworks/module-sarp-hyva requires aheadworks/module-sarp3 which is NOT in
            # composer require section so I was made to implement this dirty hack
            # I am really sorry
            if os.getenv("BITBUCKET_REPO_SLUG") == "module-sarp-hyva":
                hacky_proc = subprocess.run(['php', '-d', 'memory_limit=4G', '/usr/local/bin/composer', 'require', '--prefer-dist',
                                    'aheadworks/module-sarp3'])

            proc = subprocess.run(['php', '-d', 'memory_limit=4G', '/usr/local/bin/composer', 'require', '--prefer-dist',
                                   '{e[name]}:{e[version]}'.format(e=composer)])

            if proc.returncode:
                raise RuntimeError("Failed to install extension")

        # we test modules that are downloaded by composer require in /var/www/html/vendor/aheadworks/module_name
        # path of module is linked by composer with path in where we started our test step in bitbucket pipeline
        result_path = self.BASIC_PATH / 'vendor' / composer['name']
        return result_path

    def codesniffer(self, path, report_file, severity=9, report="junit"):
        """Run codesniffer tests against the path"""

        if os.getenv('SEVERITY'):
            severity = os.getenv('SEVERITY')
        self.assert_path_exists(path)

        allowed_report_formats = ["full", "xml", "checkstyle", "csv",
                                   "json", "junit", "emacs", "source",
                                   "summary", "diff", "svnblame",
                                   "gitblame",
                                   "hgblame", "notifysend"]
        if report not in allowed_report_formats:
            print(f"Unknown report format {format}", report)
            exit(1)

        options = [
            _(self.BASIC_PATH, 'vendor/bin/phpcs'),
            path
        ]

        options += ['--severity=%s' % severity]
        options += ['--standard=Magento2']
        options += ['--extensions=php,phtml']

        if report_file:
            options += ['--report=' + report]
            stdout = open(report_file, 'w')

        process = subprocess.run(options, stdout=None)
        exit(process.returncode)


    def unit(self, path, report_file, report="junit"):
        """Run unit tests for extension at path"""

        self.assert_path_exists(path)
        result_path = self.install(path)
        time.sleep(3)
        self.di_compile()
        time.sleep(3)

        try:
            os.mkdir('allure')
            shutil.copyfile("/var/www/html/dev/tests/unit/allure/allure.config.php", "allure/allure.config.php")
        except:
            print('allure config not found, its unit test for 2.4.4')

        options = [
            _(self.BASIC_PATH, 'vendor/bin/phpunit'),
            '--configuration', _(self.BASIC_PATH, 'dev/tests/unit/phpunit.xml.dist')
        ]

        if report_file:
            options += ['--log-%s' % report, report_file]

        proc = subprocess.Popen(options + [_(result_path, 'Test/Unit')])
        proc.communicate()

        exit(proc.returncode)


    def phpstan(self, path, report_file, severity=0):
        """Run phpstan static analysis against the path"""

        self.assert_path_exists(path)
        result_path = self.install(path)
        self.di_compile()

        options = [
            _(self.BASIC_PATH, 'vendor/bin/phpstan'),
            'analyse',
            path
        ]
        config = _(path, 'phpstan.neon')
        if pathlib.Path(config).is_file():
            options += ['--configuration', config]

        if os.getenv('PHPSTANSEVERITY'):
            severity = os.getenv('PHPSTANSEVERITY')

        options += ['--level', str(severity)]
        options += [
            '--autoload-file',
            _(self.BASIC_PATH, 'dev/tests/integration/framework/autoload.php')
        ]

        if report_file:
            stdout = open(report_file, 'w')

        process = subprocess.run(options, stdout=None)
        print(process)
        os.system("cat test-results/*")
        exit(process.returncode)


    def mess_detector(self, path, report_file=False, report="xml"):
        """Run mess detector against the path"""

        self.assert_path_exists(path)
        result_path = self.install(path)
        self.di_compile()

        if not report_file:
            report = 'ansi'

        options = [
            _(self.BASIC_PATH, 'vendor/bin/phpmd'),
            result_path,
            report,
            _(self.BASIC_PATH, 'dev/tests/static/testsuite/Magento/Test/Php/_files/phpmd/ruleset.xml')
        ]

        if report_file:
            stdout = open(report_file, 'w')

        process = subprocess.run(options, stdout=None)
        exit(process.returncode)


    def install_magento(self, path):
        """Run install magento with extension at path"""

        self.assert_path_exists(path)

        try:
            output = subprocess.run(
                ['sh', "/tmp/install-db-and-magento.sh"],
                check=True,
                text=True
            )
        except subprocess.CalledProcessError as e:
            os.system("mkdir logs1")
            # user pass and db_name as variable in install-db-and-magento.sh
            os.system("mysqldump -umagento2 -pPasswor8 magento_db >> logs1/dump.sql")
            os.system(f"cp /var/www/html/var/log/* logs1")
            os.system("cp /var/www/html/app/etc/* logs1")
            exit(e.returncode)

        result_path = self.install(path)
        self.di_compile()

        exit(output.returncode)



from aheadworks_core.model.http.api_request import ApiRequest as Api
from jira import JIRA
import json


class JiraApiManager:
    """api manager for jira"""

    def __init__(self, config):
        self.config = config
        self.request = Api(config=self.config)
        self.jira = JIRA(
            server=self.config.url,
            basic_auth=(self.config.user, self.config.token)
        )

    def get_jira_instance(self):
        return self.jira

    def get_issue_url(self, task_key):
        return '{}/browse/{}'.format(self.config.url, task_key)

    def get_release_report_all_issues_url(self, project_key, version_id):
        return '{}/projects/{}/versions/{}/tab/release-report-all-issues'.format(
            self.config.url,
            project_key,
            version_id
        )




class JiraConfig:
    def __init__(self, user, token, url=None):
        if url is None:
            self.url = 'https://aheadworks.atlassian.net'

        self.auth_type = 'auth'
        self.user = user
        self.token = token
